@@ -1,10 +1,11 @@
 package Net::DNS::DomainName;
+use base qw(Net::DNS::Domain);
 
 #
-# $Id: DomainName.pm 1177 2014-03-10 08:33:21Z willem $
+# $Id: DomainName.pm 953 2011-11-02 21:15:28Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1177 $)[1];
+$VERSION = (qw$LastChangedRevision: 953 $)[1];
 
 
 =head1 NAME
@@ -40,10 +41,6 @@ introduced by RFC3597.
 
 
 use strict;
-use base qw(Net::DNS::Domain);
-
-use constant OKlc => eval { require 5.010; } || 0;
-
 use integer;
 use Carp;
 
@@ -58,20 +55,6 @@ Creates a domain name object which identifies the domain specified
 by the character string argument.
 
 
-=head2 canonical
-
-    $data = $object->canonical;
-
-Returns the canonical wire-format representation of the domain name
-as defined in RFC2535(8.1).
-
-=cut
-
-sub canonical {
-	join '', map pack( 'C a*', length($_), OKlc ? lc($_) : _lc($_) ), shift->_wire, '';
-}
-
-
 =head2 decode
 
     $object = decode Net::DNS::DomainName( \$buffer, $offset, $hash );
@@ -93,8 +76,7 @@ data buffer.
 =cut
 
 sub decode {
-	my $label  = [];
-	my $self   = bless {label => $label}, shift;
+	my $self   = bless {}, shift;
 	my $buffer = shift;					# reference to data buffer
 	my $offset = shift || 0;				# offset within buffer
 	my $cache  = shift || {};				# hashed objectref by offset
@@ -108,7 +90,7 @@ sub decode {
 			return wantarray ? ( $self, ++$index ) : $self;
 
 		} elsif ( $header < 0x40 ) {			# non-terminal label
-			push @$label, substr( $$buffer, ++$index, $header );
+			push( @{$self->{label}}, substr( $$buffer, ++$index, $header ) );
 			$index += $header;
 
 		} elsif ( $header < 0xC0 ) {			# deprecated extended label types
@@ -141,18 +123,12 @@ sub encode {
 
 ########################################
 
-sub _lc {			## work around 5.8.x case-folding bug
-	( my $s = shift ) =~ tr [\101-\132] [\141-\172];
-	return $s;
-}
-
-sub _wire {			## Generate list of wire-format labels
+sub _wire {				## Generate list of wire-format labels
 	my $self = shift;
 
-	my $label = $self->{label} || [];
-	my @label = @$label;
-	my $origin = $self->{origin} || return (@label);
-	return ( @label, $origin->_wire );
+	my @label = @{$self->{label}} if $self->{label};
+	my @suffx = $self->{origin}->_wire if $self->{origin};
+	return ( @label, @suffx );
 }
 
 
@@ -197,24 +173,33 @@ uncompressed canonical representation defined in RFC2535(8.1).
 sub encode {
 	my $self   = shift;
 	my $offset = shift || 0;				# offset in data buffer
-	my $hash   = shift || return $self->canonical;		# hashed offset by name
+	my $hash   = shift;					# hashed offset by name
+
+	return join '', map pack( 'C a*', length($_), _lc($_) ), $self->_wire, '' unless defined $hash;
 
 	my @labels = $self->_wire;
 	my $data   = '';
 	while (@labels) {
 		my $name = join( '.', @labels );
 
-		return $data . pack( 'n', 0xC000 | $hash->{$name} ) if defined $hash->{$name};
+		return $data . pack( 'n', $hash->{$name} ) if defined $hash->{$name};
 
 		my $label  = shift @labels;
 		my $length = length $label;
 		$data .= pack( 'C a*', $length, $label );
 
 		next unless $offset < 0x4000;
-		$hash->{$name} = $offset;
+		$hash->{$name} = 0xC000 | $offset;
 		$offset += 1 + $length;
 	}
-	$data .= pack 'x';
+	$data .= chr(0);
+}
+
+
+sub _lc {
+	local $_ = shift;
+	tr [\101-\132] [\141-\172];
+	return $_;
 }
 
 
@@ -253,10 +238,21 @@ canonical form defined in RFC2535(8.1).
 =cut
 
 sub encode {
-	return shift->canonical unless defined $_[2];
-	join '', map pack( 'C a*', length($_), $_ ), shift->_wire, '';
+	my ( $self, $offset, $hash ) = @_;
+
+	return join '', map pack( 'C a*', length($_), $_ ), $self->_wire, '' if defined $hash;
+
+	return join '', map pack( 'C a*', length($_), _lc($_) ), $self->_wire, '';
 }
 
+
+sub _lc {
+	local $_ = shift;
+	tr [\101-\132] [\141-\172];
+	return $_;
+}
+
+
 1;
 __END__
 
