@@ -1,18 +1,18 @@
 package Net::DNS::Question;
 #
-# $Id: Question.pm 690 2007-12-27 20:39:40Z olaf $
+# $Id: Question.pm 932 2011-10-26 12:40:48Z willem $
 #
 use strict;
-BEGIN { 
+BEGIN {
     eval { require bytes; }
-} 
+}
 
 use vars qw($VERSION $AUTOLOAD);
 
 use Carp;
 use Net::DNS;
 
-$VERSION = (qw$LastChangedRevision: 690 $)[1];
+$VERSION = (qw$LastChangedRevision: 800$)[1];
 
 =head1 NAME
 
@@ -42,34 +42,38 @@ queries in in-addr.arpa and ip6.arpa subdomains.
 =cut
 
 sub new {
-	my $class = shift;
-
-	my $qname = shift;
-	my $qtype = uc shift || 'A';
-	my $qclass = uc shift || 'IN';
+	my $self   = bless {}, shift;
+	my $qname  = shift;
+	my $qtype  = uc (shift || '');
+	my $qclass = uc (shift || '');
 
 	$qname = '' unless defined $qname;	# || ''; is NOT same!
 	$qname =~ s/\.+$//o;			# strip gratuitous trailing dot
 
-	# Check if the caller has the type and class reversed.
-	# We are not that kind for unknown types.... :-)
-	($qtype, $qclass) = ($qclass, $qtype)
-		if exists $Net::DNS::classesbyname{$qtype}
-		and exists $Net::DNS::typesbyname{$qclass};
+	# tolerate (possibly unknown) type and class in zone file order
+	unless ( exists $Net::DNS::classesbyname{$qclass} ) {
+		( $qtype, $qclass ) = ( $qclass, $qtype )
+				if exists $Net::DNS::classesbyname{$qtype};
+		( $qtype, $qclass ) = ( $qclass, $qtype ) if $qtype =~ /CLASS/;
+	}
+	unless ( exists $Net::DNS::typesbyname{$qtype} ) {
+		( $qtype, $qclass ) = ( $qclass, $qtype )
+				if exists $Net::DNS::typesbyname{$qclass};
+		( $qtype, $qclass ) = ( $qclass, $qtype ) if $qclass =~ /TYPE/;
+	}
 
 	# if argument is an IP address, do appropriate reverse lookup
 	my $reverse = _dns_addr($qname) if $qname =~ m/:|\d$/o;
 	if ( $reverse ) {
 		$qname = $reverse;
-		$qtype = 'PTR' if $qtype =~ m/^(A|AAAA)$/o;
+		$qtype ||= 'PTR';
 	}
 
-	my $self = {	qname	=> $qname,
-			qtype	=> $qtype,
-			qclass	=> $qclass
-			};
+	$self->{qname}  = $qname;
+	$self->{qtype}  = ( $qtype || 'A' );
+	$self->{qclass} = ( $qclass || 'IN' );
 
-	bless $self, $class;
+	return $self;
 }
 
 
@@ -111,18 +115,20 @@ The second argument is the offset within the packet where the question record be
 
 Returns a Net::DNS::Question object and the offset of the next location in the packet.
 
-Returns undef if the question object could not be created (e.g., corrupt or insufficient data).
+Parsing is aborted if the question object cannot be created (e.g., corrupt or insufficient data).
 
 =cut
 
+use constant PACKED_LENGTH => length pack 'n2', (0)x2;
+
 sub parse {
 	my ($class, $data, $offset) = @_;
 
 	my ($qname, $index) = Net::DNS::Packet::dn_expand($data, $offset);
-	return undef unless defined $qname;
+	die 'Exception: corrupt or incomplete data' unless $index;
 
-	my $next = $index + 4;		# + length pack('n2', 1, 2);
-	return undef if length $$data < $next;
+	my $next = $index + PACKED_LENGTH;
+	die 'Exception: incomplete data' if length $$data < $next;
 	my ($qtype, $qclass) = unpack("\@$index n2", $$data);
 
 	my $self = {	qname	=> $qname,
@@ -168,16 +174,6 @@ known as C<zclass> and refers to the zone's class.
 
 =cut
 
-sub qname  {
-	my $self = shift;
-
-	return $self->{qname} unless @_;
-
-	my $qname = shift;
-	$qname =~ s/\.+$//o if defined ($qname);			# strip gratuitous trailing dot
-	$self->{qname} = $qname;
-}
-
 sub zname  { &qname;  }
 sub ztype  { &qtype;  }
 sub zclass { &qclass; }
@@ -191,9 +187,11 @@ sub AUTOLOAD {
 
 	croak "$AUTOLOAD: no such method" unless exists $self->{$name};
 
-	return $self->{$name} = shift if @_;
+	return $self->{$name} unless @_;
 
-	return $self->{$name};
+	my $value = shift;
+	$value =~ s/\.+$//o if defined $value;	# strip gratuitous trailing dot
+	$self->{$name} = $value;
 }
 
 =head2 print
@@ -204,7 +202,7 @@ Prints the question record on the standard output.
 
 =cut
 
-sub print {	print shift->string, "\n"; }
+sub print {	print &string, "\n"; }
 
 =head2 string
 
@@ -244,16 +242,23 @@ sub data {
 	return $data;
 }
 
+
+1;
+__END__
+
+
 =head1 COPYRIGHT
 
-Copyright (c) 1997-2002 Michael Fuhr. 
+Copyright (c) 1997-2002 Michael Fuhr.
 
 Portions Copyright (c) 2002-2004 Chris Reinhardt.
 
-Portions Copyright (c) 2003,2006-2007 Dick Franks.
+Portions Copyright (c) 2003,2006-2009 Dick Franks.
 
-All rights reserved.  This program is free software; you may redistribute
-it and/or modify it under the same terms as Perl itself.
+All rights reserved.
+
+This program is free software; you may redistribute it and/or
+modify it under the same terms as Perl itself.
 
 =head1 SEE ALSO
 
@@ -263,4 +268,3 @@ RFC 1035 Section 4.1.2
 
 =cut
 
-1;
